# Copyright (C) 2009 Canonical, Ltd.
#
# This library is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License
# version 3.0 as published by the Free Software Foundation.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License version 3.0 for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library. If not, see
# <http://www.gnu.org/licenses/>.
#
# Authored by Stuart Langridge <stuart.langridge@canonical.com>

 
import gtk, gobject, os
import rb, rhythmdb
try:
    import U1MusicStore
    using_prototype = False
except ImportError:
    from pythonprototype import U1MusicStorePythonPrototype as U1MusicStore
    using_prototype = True
    

class U1MusicStorePlugin (rb.Plugin):
    def __init__(self):
        rb.Plugin.__init__(self)
    def activate(self, shell):
        self.db = shell.get_property("db")
        group = rb.rb_source_group_get_by_name ("stores")
        if not group:
                group = rb.rb_source_group_register ("stores",
                                                     _("Stores"),
                                                     rb.SOURCE_GROUP_CATEGORY_FIXED)
        theme = gtk.icon_theme_get_default()
        rb.append_plugin_source_path(theme, "/icons")
        width, height = gtk.icon_size_lookup(gtk.ICON_SIZE_LARGE_TOOLBAR)
        icon = rb.try_load_icon(theme, "ubuntuone-client-working", width, 0)
        self.entry_type = self.db.entry_register_type("U1EntryType")
        self.source = gobject.new (U1Source,
                                   shell=shell,
                                   entry_type=self.entry_type,
                                   source_group=group,
                                   icon=icon,
                                   plugin=self)
        shell.register_entry_type_for_source(self.source, self.entry_type)
        shell.append_source(self.source, None) # Add the source to the list
        self.shell = shell
        self.source.connect("preview-mp3", self.play_preview_mp3)
    def deactivate(self, shell):
        pass
    def play_preview_mp3(self, source, url, title):
        # create an entry, don't save it, and play it
        entry = self.shell.props.db.entry_lookup_by_location(url)
        if entry is None:
            entry = self.shell.props.db.entry_new(self.entry_type, url)
        self.shell.props.db.set(entry, rhythmdb.PROP_TITLE, title)
        player = self.shell.get_player()
        player.stop()
        player.play()
        player.play_entry(entry, self.source)
        # FIXME delete this entry when it finishes playing. Don't know how yet.

class U1Source(rb.Source):
    # gproperties required so that rb.Source is instantiable
    __gproperties__ = {
      'plugin': (rb.Plugin, 'plugin', 'plugin', 
                    gobject.PARAM_WRITABLE|gobject.PARAM_CONSTRUCT_ONLY),
    }
    # we have the preview-mp3 signal; we receive it from the widget, and re-emit
    # it so that the plugin gets it, because the plugin actually plays the mp3
    __gsignals__ = {
      "preview-mp3": (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (str, str)),
    }
    
    def __init__(self):
        rb.Source.__init__(self, name=_("Ubuntu One"))
        self.__activated = False
    def do_impl_activate(self):
        if self.__activated: return
        self.__activated = True
        self.browser = U1MusicStore()
        sc = gtk.ScrolledWindow()
        if using_prototype:
            storeurl = os.environ.get("U1MUSICSTOREURL", None)
            vbox = gtk.VBox()
            if storeurl:
                labeltext = ("WARNING: You are using the Python "
                    "prototype of the music store widget!")
            else:
                labeltext = ("WARNING: You are using the Python "
                    "prototype of the music store widget; it is assumed that "
                    "you're doing this for testing, but you have not "
                    "specified U1MUSICSTOREURL on the command line. Re-run "
                    "Rhythmbox as 'U1MUSICSTOREURL=http://localhost:xxxxx/music/store "
                    "rhythmbox -D U1'.")
            lbl = gtk.Label(labeltext)
            lbl.set_line_wrap(True)
            vbox.pack_start(lbl, expand=False, fill=False)
            vbox.pack_start(self.browser, expand=True, fill=True)
            sc.add(vbox)
        else:
            sc.add(self.browser)
        sc.show_all()
        self.pack_start(sc)
        self.browser.set_no_show_all(True)
        self.browser.set_property("visible", True)
        rb.Source.do_impl_activate (self)
        self.browser.connect("preview-mp3", self.re_emit_preview)
    def do_impl_can_pause(self): return True # so we can pause, else we segfault
    def re_emit_preview(self, widget, url, title):
        self.emit("preview-mp3", url, title)
    def do_set_property(self, property, value):
        if property.name == 'plugin':
            self.__plugin = value
        else:
            raise AttributeError, 'unknown property %s' % property.name


