import webkit, gobject, os, urlparse
from oauth import oauth

__version__ = "0.1"

class U1MusicStorePythonPrototype(webkit.WebView):
    __gsignals__ = {
      "preview-mp3": (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (str, str)),
    }
    def __init__(self):
        super(U1MusicStorePythonPrototype, self).__init__()
        self.set_size_request(839,300) # design is 839px wide
        
        settings = self.get_settings()
        settings.props.user_agent = "U1MusicStore widget (%s)" % __version__
        self.set_settings(settings)
        
        self.connect("load-finished", self.load_finished)
        self.connect("status-bar-text-changed", self.status_bar)
        self.connect("load-error", self.load_error)
        url = os.environ.get("U1MUSICSTOREURL", 
            "http://one.ubuntu.com/music/login")
        # oauth sign URL
        if url.startswith("http://localhost") or url.startswith("http://127.0.0.1"):
            # we are testing; get a test token
            oauthfile=os.environ.get("OAUTHKEYFILE", None)
            if not oauthfile:
                self.load_string("fail! if you specify U1MUSICSTOREURL you must "
                "also specify OAUTHKEYFILE as a file with oauth keys in it", 
                "text/plain", "utf-8", "file:///")
                return
            fp = open(oauthfile)
            oauthstring = fp.read()
            fp.close()
            token = oauth.OAuthToken.from_string(oauthstring)
        else:
            # we are not testing; get a token from the keyring
            token = oauth.OAuthToken.from_string("oauth_token=fail&oauth_token_secret=fail")
        # sign URL with token
        consumer = oauth.OAuthConsumer("ubuntuone", "hammertime")
        rq = oauth.OAuthRequest.from_consumer_and_token(http_url=url,
            http_method="GET", oauth_consumer=consumer, token=token)
        rq.sign_request(oauth.OAuthSignatureMethod_HMAC_SHA1(), consumer, token)
        self.open(rq.to_url())
    def load_finished(self, view, frame):
        "Inject JavaScript to enable previews and hide the Flash player"
        self.execute_script("""
(function() {
    // capture the preview
    if (playTrack) { 
        playTrack = function(file, title) { 
            window.status = "u1preview:::" + file + ":::" + title;
        }
    }
    // and hide the flashplayer
    var ply = document.getElementById("flashPlayer");
    if (ply) {
        ply.parentNode.removeChild(ply);
    }
})()""")
        # Allow us to specifically override the library page links
        library_override = os.environ.get("U1MUSICLIBRARYURL", None)
        if library_override:
            self.execute_script("""
            (function(){
                var lnks = document.querySelectorAll("a");
                for (var i=0; i<lnks.length; i++) {
                    if (lnks[i].href.indexOf("library.aspx") != -1) {
                        lnks[i].href = "%s";
                    }
                }
            })()
            """ % library_override)
    def status_bar(self, view, msg):
        if msg.startswith("u1preview:::"):
            cmd, url, title = msg.split(":::")
            self.emit("preview-mp3", urlparse.urljoin(self.props.uri, url), title)
    def load_error(self, view, frame, url, ptr):
        self.load_string('<html><body>custom u1 error page</body></html>', 'text/html', 'utf-8', 'file:///')
        return True # handle signal so it doesn't propagate
        # FIXME does not get called for 504 Gateway Time-out; catch this error!
